/*
 * DefaultGrid.java
 *
 * Created on 17 dicembre 2007, 22.18
 *
 */

package polyliner.domain.impl.point.simple;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import polyliner.domain.exceptions.PointNotFoundException;
import polyliner.domain.interfaces.FilterCondition;
import polyliner.domain.interfaces.Grid;
import polyliner.domain.interfaces.GridStateChangeListener;
import polyliner.domain.interfaces.Point;
import polyliner.domain.interfaces.PointFactory;
import polyliner.domain.interfaces.PointStateChangeListener;

/**
 *
 * @author Dario Lucia
 */
public class DefaultGrid implements Grid,PointStateChangeListener {
    
    private double width;
    private double height;
    
    private PointFactory factory;
    
    protected Map<Integer,Point> points;
    
    private List<GridStateChangeListener> listeners;
    
    /** Creates a new instance of DefaultGrid */
    public DefaultGrid(double width, double height) {
        this(width,height,new DefaultPointFactory());
    }
    
    /** Creates a new instance of DefaultGrid */
    public DefaultGrid(double width, double height, PointFactory factory) {
        this.width = width;
        this.height = height;
        this.factory = factory;
        this.points = new TreeMap<Integer,Point>();
        this.listeners = new ArrayList<GridStateChangeListener>();
    }

    public Point addPoint(double x, double y) {
        Point p = this.factory.newPoint(x,y);
        this.points.put(p.getId(),p);
        p.addStateChangeListener(this);
        addPointUpdate(p);
        return p;
    }

    public Point removePoint(int id) throws PointNotFoundException {
        Point p = this.points.remove(id);
        if(p != null) {
            p.removeStateChangeListener(this);
            removePointUpdate(p); 
        } else  
            throw new PointNotFoundException(id);
        return p;
    }

    public void removeAll() {
        Integer[] ids = this.points.keySet().toArray(new Integer[0]);
        Point p;
        for(Integer id : ids) {
            p = this.points.remove(id); 
            p.removeStateChangeListener(this);
            removePointUpdate(p); 
        }
        
    }

    public Point getPoint(int id) throws PointNotFoundException {
        Point p = this.points.get(id);
        if(p == null)
            throw new PointNotFoundException(id);
        return p;
    }

    public Point[] getPoints() {
        return this.points.values().toArray(new Point[this.points.size()]);
    }

    public Point[] getPoints(FilterCondition c) {
        List<Point> tempList = new ArrayList<Point>();
        for(Point p : this.points.values())
            if(c.isSatisfiedBy(p)) 
                tempList.add(p);
        return tempList.toArray(new Point[tempList.size()]);
    }

    public double getWidth() {
        return this.width;
    }

    public double getHeight() {
        return this.height;
    }

    public void addStateChangeListener(GridStateChangeListener listener) {
        if(!this.listeners.contains(listener))
            this.listeners.add(listener);
    }

    public void removeStateChangeListener(GridStateChangeListener listener) {
        if(this.listeners.contains(listener))
            this.listeners.remove(listener);
    }
    
    protected void addPointUpdate(Point p) {
        for(GridStateChangeListener l : this.listeners)
            l.pointAdded(this,p);
        stateChangeUpdate();
    }
    
    protected void removePointUpdate(Point p) {
        for(GridStateChangeListener l : this.listeners)
            l.pointRemoved(this,p);
        stateChangeUpdate();
    }
    
    protected void stateChangeUpdate() {
        for(GridStateChangeListener l : this.listeners)
            l.stateChanged(this);
    }
    
    protected void setPointFactory(PointFactory factory) {
        this.factory = factory;
    }

    public void stateChanged(Point p) {
        stateChangeUpdate();
    }
    
}
