/*
 * ScaleGrid.java
 *
 * Created on 22 dicembre 2007, 10.55
 *
 */

package polyliner.domain.utilities.scale;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import polyliner.domain.exceptions.PointNotFoundException;
import polyliner.domain.interfaces.FilterCondition;
import polyliner.domain.interfaces.Grid;
import polyliner.domain.interfaces.GridStateChangeListener;
import polyliner.domain.interfaces.Point;

/**
 * Decorator class for implementations of Grid
 *
 * @author Dario Lucia
 */
public class ScaleGrid implements Grid,GridStateChangeListener {
    
    private Grid baseGrid;
    
    private double scaleFactor;
    
    private Map<Integer,ScalePoint> points;
    private List<GridStateChangeListener> listeners;
    
    /** Creates a new instance of ScaleGrid */
    public ScaleGrid(Grid base,double scaleFactor) {
        this.baseGrid = base;
        this.scaleFactor = scaleFactor;
        this.points = new TreeMap<Integer,ScalePoint>();
        this.listeners = new ArrayList<GridStateChangeListener>();
        this.baseGrid.addStateChangeListener(this);
    }

    public Point addPoint(double x, double y) {
        Point p = this.baseGrid.addPoint(x/this.scaleFactor,y/this.scaleFactor);
        ScalePoint sp = new ScalePoint(p,this.scaleFactor);
        this.points.put(p.getId(),sp);
        
        addPointUpdate(sp);
        return sp;
    }

    public Point removePoint(int id) throws PointNotFoundException {
        Point p = this.baseGrid.removePoint(id);
        ScalePoint sp = this.points.remove(id);
        if(sp == null) throw new PointNotFoundException(id);
        
        removePointUpdate(sp);
        return sp;
    }

    public void removeAll() {
        this.baseGrid.removeAll();
        Integer[] ids = this.points.keySet().toArray(new Integer[0]);
        Point p;
        for(Integer id : ids) {
            p = this.points.remove(id); 
            
            removePointUpdate(p); 
        }
    }

    public Point getPoint(int id) throws PointNotFoundException {
        Point p = this.points.get(id);
        if(p == null)
            throw new PointNotFoundException(id);
        return p;
    }

    public Point[] getPoints() {
        for(Point p : this.points.values()) {
            System.out.println("EXP - Point id:" + p.getId());
        }
        return this.points.values().toArray(new Point[this.points.values().size()]);
        
    }

    public Point[] getPoints(FilterCondition c) {
        List<Point> tempList = new ArrayList<Point>();
        for(Point p : this.points.values())
            if(c.isSatisfiedBy(p)) 
                tempList.add(p);
        return tempList.toArray(new Point[tempList.size()]);
    }

    public double getWidth() {
        return this.scaleFactor * this.baseGrid.getWidth();
    }

    public double getHeight() {
        return this.scaleFactor * this.baseGrid.getHeight();
    }

    public void addStateChangeListener(GridStateChangeListener listener) {
        if(!this.listeners.contains(listener))
            this.listeners.add(listener);
    }

    public void removeStateChangeListener(GridStateChangeListener listener) {
        if(this.listeners.contains(listener))
            this.listeners.remove(listener);
    }
    
    protected void addPointUpdate(Point p) {
        
        for(GridStateChangeListener l : this.listeners)
            l.pointAdded(this,p);
       
    }
    
    protected void removePointUpdate(Point p) {
        
        for(GridStateChangeListener l : this.listeners)
            l.pointRemoved(this,p);
        
    }
    
    protected void stateChangeUpdate() {
        
        for(GridStateChangeListener l : this.listeners)
            l.stateChanged(this);
    }
    
    public double getScaleFactor() {
        return this.scaleFactor;
    }
    
    public void setScaleFactor(double scaleFactor) {
        // Instance variable
        this.scaleFactor = scaleFactor;
        // All points
        for(ScalePoint p : this.points.values()) {
            p.setScaleFactor(scaleFactor);
        }
        stateChangeUpdate();
    }

    public void pointAdded(Grid grid, Point p) {
        // Do nothing
    }

    public void pointRemoved(Grid grid, Point p) {
        // Do nothing
    }

    public void stateChanged(Grid grid) {
        stateChangeUpdate();
    }

}
